import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, EntityEquippableComponent, ItemDurabilityComponent, GameMode} from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

const settingsOrder = [
  "maxVeinSpeed",
  "maxVeinBlocks",
  "veinUsesDurability",
  "veinMiner",
];

const defaultValues = {
  maxVeinSpeed: 3,
  maxVeinBlocks: 16,
  veinUsesDurability: 1,
  veinMiner: 1,
};

  if (world.scoreboard.getObjective("settings") == undefined) {
    world.scoreboard.addObjective("settings", "settings");
  }

  settingsOrder.forEach((setting) => {
    const objective = world.scoreboard.getObjective("settings");
    try {
      objective.getScore(setting);
    } catch (error) {
      // Set the default value if an error occurs
      const defaultValue = defaultValues[setting] || 0; // Use 0 if not specified
      objective.setScore(setting, defaultValue);
    }
  });

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_vein:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_vein:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}
const oreTypeIds = [
  "minecraft:coal_ore",
  "minecraft:deepslate_coal_ore",
  "minecraft:iron_ore",
  "minecraft:deepslate_iron_ore",
  "minecraft:gold_ore",
  "minecraft:deepslate_gold_ore",
  "minecraft:lapis_ore",
  "minecraft:deepslate_lapis_ore",
  "minecraft:diamond_ore",
  "minecraft:deepslate_diamond_ore",
  "minecraft:redstone_ore",
  "minecraft:lit_redstone_ore",
  "minecraft:deepslate_redstone_ore",
  "minecraft:lit_deepslate_redstone_ore",
  "minecraft:emerald_ore",
  "minecraft:deepslate_emerald_ore",
  "minecraft:copper_ore",
  "minecraft:deepslate_copper_ore",
  "minecraft:nether_quartz_ore",
  "minecraft:ancient_debris",
  // Add any other ore type IDs here
];

world.beforeEvents.playerBreakBlock.subscribe((event) => {
  let maxVeinBlocks = world.scoreboard
    .getObjective("settings")
    .getScore("maxVeinBlocks"); // Example value, you can change this dynamically
  let blockBreakDelay = world.scoreboard
    .getObjective("settings")
    .getScore("maxVeinSpeed"); // Delay in ticks between block breaks
  const player = event.player;
  const block = event.block;
  const equippable = player.getComponent("equippable");
  const item = equippable.getEquipment(EquipmentSlot.Mainhand);
  const playerEquipment = player.getComponent(
    EntityEquippableComponent.componentId
  );
  const durabilityComponent = item.getComponent(
    ItemDurabilityComponent.componentId
  );

  // Check if the block is an ore before starting vein mining
  if (
    oreTypeIds.includes(block.typeId) &&
    item &&
    item.hasTag("minecraft:is_pickaxe")
  ) {
    // Start mining with a delay
    system.runTimeout(
      () =>
        mineVein(
          block,
          player,
          player.dimension,
          maxVeinBlocks,
          blockBreakDelay,
          item,
          durabilityComponent,
          playerEquipment
        ),
      blockBreakDelay
    );
  }
});

function mineVein(
  initialBlock,
  player,
  dimension,
  maxBlocks,
  delay,
  item,
  durabilityComponent,
  playerEquipment
) {
  let blocksToMine = [initialBlock];
  let minedBlocks = 0;

  function mineNextBlock() {
    if (blocksToMine.length > 0 && minedBlocks < maxBlocks && item) {
      const currentBlock = blocksToMine.shift();
      const neighboringOres = getNeighboringOres(currentBlock, dimension);

      neighboringOres.forEach((oreBlock) => {
        if (minedBlocks < maxBlocks && item) {
          blocksToMine.push(oreBlock);
          player.runCommand(
            `setblock ${oreBlock.location.x} ${oreBlock.location.y} ${oreBlock.location.z} air destroy`
          );

          if (player.getGameMode() !== GameMode.creative) {
            // Check durability before incrementing
            if (
              durabilityComponent.damage < durabilityComponent.maxDurability &&
              world.scoreboard
                .getObjective("settings")
                .getScore("veinUsesDurability")
            ) {
              durabilityComponent.damage++;
            }
            if (
              durabilityComponent.damage >= durabilityComponent.maxDurability
            ) {
              player.runCommand("playsound random.break @a ~ ~ ~");
              playerEquipment.setEquipment(EquipmentSlot.Mainhand, null);
              item = null; // Set item to null to stop further mining
              return; // Exit immediately to prevent further execution
            } else {
              playerEquipment.setEquipment(EquipmentSlot.Mainhand, item);
            }
          }

          minedBlocks++;
        }
      });

      if (item) {
        // Only continue if the item is not broken
        system.runTimeout(() => mineNextBlock(), delay);
      }
    }
  }

  mineNextBlock(); // Start the mining process
}

function getNeighboringOres(block, dimension) {
  const offsets = [
    { x: 1, y: 0, z: 0 },
    { x: -1, y: 0, z: 0 },
    { x: 0, y: 1, z: 0 },
    { x: 0, y: -1, z: 0 },
    { x: 0, z: 1, y: 0 },
    { x: 0, z: -1, y: 0 },
    { x: 1, y: 1, z: 0 },
    { x: -1, y: 1, z: 0 },
    { x: 1, y: -1, z: 0 },
    { x: -1, y: -1, z: 0 },
    { x: 0, y: 1, z: 1 },
    { x: 0, y: -1, z: 1 },
    { x: 0, y: 1, z: -1 },
    { x: 0, y: -1, z: -1 },
    { x: 1, y: 0, z: 1 },
    { x: -1, y: 0, z: 1 },
    { x: 1, y: 0, z: -1 },
    { x: -1, y: 0, z: -1 },
    { x: 1, y: 1, z: 1 },
    { x: -1, y: 1, z: 1 },
    { x: 1, y: -1, z: 1 },
    { x: -1, y: -1, z: 1 },
    { x: 1, y: 1, z: -1 },
    { x: -1, y: 1, z: -1 },
    { x: 1, y: -1, z: -1 },
    { x: -1, y: -1, z: -1 },
  ];

  let neighboringOres = [];

  offsets.forEach((offset) => {
    const neighbor = dimension.getBlock({
      x: block.location.x + offset.x,
      y: block.location.y + offset.y,
      z: block.location.z + offset.z,
    });

    if (neighbor && oreTypeIds.includes(neighbor.typeId)) {
      neighboringOres.push(neighbor);
    }
  });

  return neighboringOres;
}


















world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_vein_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_vein_version", VERSION);
    if (player.hasTag("sg_vein_received_guide")) {
      player.removeTag("sg_vein_received_guide");
      player.sendMessage({
        translate: "sg_vein.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_vein:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_vein_version", VERSION);
    player.sendMessage({
      translate: "sg_vein.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // OPTIONS
    translation_key: "sg_vein.guide.options.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {

      const objective = world.scoreboard.getObjective("settings");

      // Create and show the modal form
      const form = new ModalFormData()
        .title("Options")
        .slider(
          "Vein Mining Speed",
          0,
          10,
          1,
          objective.getScore("maxVeinSpeed")
        )
        .slider("Max Blocks", 1, 32, 1, objective.getScore("maxVeinBlocks"))
        .toggle(
          "Uses Durability",
          Boolean(objective.getScore("veinUsesDurability"))
        )
        .toggle("Enabled", Boolean(objective.getScore("veinMiner")))
        
      form.show(player).then((res) => {
        if (res.canceled) return;

        // Iterate over form values and map them to the settings labels
        const settings = res.formValues;
        settings.forEach((value, index) => {
          const label = settingsOrder[index];
          if (label) {
            objective.setScore(label, value);
          }
        });
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_vein.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_vein.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_vein.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_vein.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_vein.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_vein.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_vein.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_vein:guide") {
      showMainPage(event.source);
    }
  });
});